#!/bin/bash

# Copyright UiPath 2021
#
# =================
# LICENSE AGREEMENT
# -----------------
#   Use of paid UiPath products and services is subject to the licensing agreement
#   executed between you and UiPath. Unless otherwise indicated by UiPath, use of free
#   UiPath products is subject to the associated licensing agreement available here:
#   https://www.uipath.com/legal/trust-and-security/legal-terms (or successor website).
#   You must not use this file separately from the product it is a part of or is associated with.

set -e

agentImageDirectory="/var/lib/rancher/rke2/agent/images"
containerdSocket="/run/k3s/containerd/containerd.sock"
ns="k8s.io"
username_file="/etc/image-cred/username"
password_file="/etc/image-cred/password"
image_file="/etc/images/longhorn-image-list"

max_attempt=5
retry=0

registry=""
[ -n "$REGISTRY_NAME" ] && registry="${REGISTRY_NAME}/"

ctr_cred=""
if [[ -f "$username_file" && -f "$password_file" ]]; then
  user=$(cat $username_file)
  pass=$(cat $password_file)
  ctr_cred="-u ${user}:${pass}"
fi

# Iterate over the list of images
while read -r img; do
  # Make tar file name
  image_name=$(echo "${img}" |  cut -d':' -f1 | tr '/' '_')
  image_tag=$(echo "${img}" | awk -F'/' '{print $NF}' | cut -d':' -f2 | tr '.' '_')
  file_name="${image_name}_${image_tag}.tar"

  # Check if image tar file already exists
  if [[ ! -f "${agentImageDirectory}"/"${file_name}" ]]; then
    echo "Image ${img} not found, pulling it"

    retry=0
    copied=0
    while [ $retry -lt $max_attempt ]; do
      retry=$((retry+1))
      rm -f "${agentImageDirectory}"/"${file_name}" || true

      # Pull the image using ctr
      #shellcheck disable=SC2086
      /var/lib/rancher/rke2/bin/ctr --address "$containerdSocket" -n "$ns" images pull "${registry}${img}" -k ${ctr_cred} || { echo "Will retry.. attempt $retry/$max_attempt" && sleep 5 && continue; }

      # Export the image using ctr
      /var/lib/rancher/rke2/bin/ctr --address "$containerdSocket" -n "$ns" images export "${agentImageDirectory}"/"${file_name}" "${registry}${img}" || { echo "Will retry.. attempt $retry/$max_attempt" && sleep 5 && continue; }
      copied=1
      break
    done
    # this will restart pod and retry all steps
    [[ $copied -eq 0 ]] && exit 1
  else
    echo "Image ${img} already exists."
  fi
done < "${image_file}"

echo "Image copy completed"
sleep infinity
