
function testArgoCDPodCount() {
    local serverAndRepoPodCount redisAndHAProxyPodCount replicaCount podCount applicationControllerCount validTestRailIds requestedTestRailIds

    serverAndRepoPodCount=$1
    redisAndHAProxyPodCount=$2
    controllerAndDexCount=1
    requestedTestRailIds=$3

    validTestRailIds=("1621328" "1621329" "1663791" "1663797")
    for validTestRailId in "${validTestRailIds[@]}"; do
        if [[ "${requestedTestRailIds}" =~ "${validTestRailId}" ]]; then
            result="false";

            replicaCount=$(fetchReplicaCount statefulset argocd argocd-application-controller)
            if [[ -z $replicaCount && -z $serverAndRepoPodCount && -z $redisAndHAProxyPodCount ]]; then
                result="true"
                appendOutput "${validTestRailId}" "${result}"
                return
            fi

            podCount=$(kubectl -n argocd get pods --field-selector=status.phase=Running --selector=app=argocd-application-controller --output json | jq -j ".items | length")
            if [[ ${podCount} -ge ${controllerAndDexCount} && ${podCount} -le ${replicaCount} ]]; then
                result="true"
            fi

            replicaCount=$(fetchReplicaCount statefulset argocd argocd-redis-ha-server)
            podCount=$(kubectl -n argocd get pods --field-selector=status.phase=Running --selector=app=argocd-redis-ha-server --output json | jq -j ".items | length")
            if [[ ${podCount} -ge ${redisAndHAProxyPodCount} && ${podCount} -le ${replicaCount} && "${result}" == "true" ]]; then
                result="true"
            fi

            replicaCount=$(fetchReplicaCount deploy argocd argocd-redis-ha-haproxy)
            podCount=$(kubectl -n argocd get pods --field-selector=status.phase=Running --selector=app=argocd-redis-ha --output json | jq -j ".items | length")
            if [[ ${podCount} -ge ${expectedWebhookPodCount} && ${podCount} -le ${replicaCount} && "${result}" == "true" ]]; then
                result="true"
            fi


            replicaCount=$(fetchReplicaCount deploy argocd argocd-server)
            podCount=$(kubectl -n argocd get pods --field-selector=status.phase=Running --selector=app.kubernetes.io/name=argocd-server --output json | jq -j ".items | length")
            if [[ ${podCount} -ge ${serverAndRepoPodCount} && ${podCount} -le ${replicaCount} && "${result}" == "true" ]]; then
                result="true"
            fi

            replicaCount=$(fetchReplicaCount deploy argocd argocd-repo-server)
            podCount=$(kubectl -n argocd get pods --field-selector=status.phase=Running --selector=app=argocd-repo-server --output json | jq -j ".items | length")
            if [[ ${podCount} -ge ${serverAndRepoPodCount} && ${podCount} -le ${replicaCount} && "${result}" == "true" ]]; then
                result="true"
            fi

            appendOutput "${validTestRailId}" "${result}"
        fi
    done
}

function setArgocdValidationParam() {
    local validationStage
    validationStage=$1
    case "$validationStage" in
    3nodeup)
        serverAndRepoPodCount=2
        redisAndHAProxyPodCount=3
        testRailIds="1621328"
        ;;
    rebootAllServers)
        serverAndRepoPodCount=2
        redisAndHAProxyPodCount=3
        testRailIds="1621329"
        ;;
    2nodeup)
        serverAndRepoPodCount=2
        redisAndHAProxyPodCount=2
        testRailIds="1663791"
       ;;
    1nodeup|0nodeup)
        serverAndRepoPodCount=""
        redisAndHAProxyPodCount=""
        testRailIds="1663797"
        ;;
    *)
        echo "Unexpected option ${validationStage} passed to ${FUNCNAME[0]}"
        exit 1
        ;;
    esac
}

function executeArgocdTest() {
    setArgocdValidationParam "${VALIDATE_STAGE}"
    testArgoCDPodCount "${serverAndRepoPodCount}" "${redisAndHAProxyPodCount}" "${testRailIds}"
}
