
function testCertManagerPodCount() {
    local expectedPodCount replicaCount podCount validTestRailIds requestedTestRailIds
    expectedPodCount=$1
    requestedTestRailIds=$2

    validTestRailIds=("1621324" "1621325" "1663796" "1663798")
    for validTestRailId in "${validTestRailIds[@]}"; do
        if [[ "${requestedTestRailIds}" =~ "${validTestRailId}" ]]; then
            result="false";
            
            replicaCount=$(fetchReplicaCount deploy cert-manager cert-manager)
            if [[ -z $replicaCount && -z $expectedPodCount ]]; then
                result="true"
                appendOutput "${validTestRailId}" "${result}"
                return
            fi
            podCount=$(kubectl -n cert-manager get pods --field-selector=status.phase=Running --selector=app=cert-manager --output json | jq -j ".items | length")
            if [[ ${podCount} -ge ${expectedPodCount} && ${podCount} -le ${replicaCount} ]]; then
                result="true"
            fi

            replicaCount=$(fetchReplicaCount deploy cert-manager cert-manager-cainjector)
            podCount=$(kubectl -n cert-manager get pods --field-selector=status.phase=Running --selector=app=cainjector --output json | jq -j ".items | length")
            if [[ ${podCount} -ge ${expectedPodCount} && ${podCount} -le ${replicaCount} && "${result}" == "true" ]]; then
                result="true"
            fi

            replicaCount=$(fetchReplicaCount deploy cert-manager cert-manager-webhook)
            podCount=$(kubectl -n cert-manager get pods --field-selector=status.phase=Running --selector=app=webhook --output json | jq -j ".items | length")
            if [[ ${podCount} -ge ${expectedPodCount} && ${podCount} -le ${replicaCount} && "${result}" == "true" ]]; then
                result="true"
            fi

            appendOutput "${validTestRailId}" "${result}"
        fi
    done
}

function setCertManagerValidationParam() {
    local validationStage
    validationStage=$1
    case "$validationStage" in
    3nodeup)
        certManagerPod=2
        testRailIds="1621324"
        ;;
    rebootAllServers)
        certManagerPod=2
        testRailIds="1621325"
        ;;
    2nodeup)
        certManagerPod=2
        testRailIds="1663796"
        ;;
    1nodeup|0nodeup)
        certManagerPod=""
        testRailIds="1663798"
        ;;
    *)
        echo "Unexpected option ${validationStage} passed to ${FUNCNAME[0]}"
        exit 1
        ;;
    esac
}

function executeCertManagerTest() {
    setCertManagerValidationParam "${VALIDATE_STAGE}"
    testCertManagerPodCount "${certManagerPod}" "${testRailIds}"
}