#!/bin/bash

ROOT_DIR="/opt/UiPathAutomationSuite/UiPath_Installer"
INPUT_FILE="/opt/UiPathAutomationSuite/UiPath_Installer/input.json"
DEFAULTS_FILE="/opt/UiPathAutomationSuite/UiPath_Installer/defaults.json"

SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)

 if [[ -z "${SCRIPT_DIR}" ]]; then
     echo "Script dir not present"
     exit 1
 fi

 #shellcheck disable=SC1090,SC1091
 source "${SCRIPT_DIR}/utils.sh"

function validateBasicChecksSuccess() {
    PRE_REQ_OUTPUT=$(${ROOT_DIR}/validateUiPathASReadiness.sh -i ${INPUT_FILE} --install-type online "$@" 2>&1)

    local test_failed="false"
    if echo "${PRE_REQ_OUTPUT}" | grep "All required packages are present";
    then
        echo "Found required packages installation"
    else
        echo "Failiure in validating required packages"
        test_failed="true"
    fi

    if echo "${PRE_REQ_OUTPUT}" | grep "Firewalld is disbaled" &&
        echo "${PRE_REQ_OUTPUT}" | grep "Service nm-cloud-setup.service not found or disabled" &&
        echo "${PRE_REQ_OUTPUT}" | grep "Service nm-cloud-setup.timer not found or disabled" &&
        echo "${PRE_REQ_OUTPUT}" | grep "Service iscsid is enbaled";
    then
        echo "Successfully verified services status"
    else
        echo "Failure in verifying services status"
        test_failed="true"
    fi

    if echo "${PRE_REQ_OUTPUT}" | grep "Server Certificate validation... DONE";
    then
        echo "Successfully validated certs status"
    else
        echo "Failure in validating certs status"
        test_failed="true"
    fi

    if echo "${PRE_REQ_OUTPUT}" | grep "Successfully resolved subdomain";
    then
        echo "Successfully validated subdomains resolution"
    else
        echo "Failure in validating subdomains resolution"
        test_failed="true"
    fi


    if echo "${PRE_REQ_OUTPUT}" | grep "CPU requirement is valid";
    then
        echo "Successfully validated CPU requirements"
    else
        echo "Failure in validating CPU requirements"
        test_failed="true"
    fi

    if echo "${PRE_REQ_OUTPUT}" | grep "Memory requirement is valid";
    then
        echo "Successfully validated Memory requirements"
    else
        echo "Failure in validating CPU requirements"
        test_failed="true"
    fi

    if [[ "${test_failed}" == "true" ]];
    then
        mark_test_failed 1658512 /tmp/test_results.json
    else
        mark_test_passed 1658512 /tmp/test_results.json
    fi
}


function validateFullPlatformChecksFailure() {
    tmpfile=$(mktemp)
    \cp $DEFAULTS_FILE  ${DEFAULTS_FILE}.bakup
    jq '.infra.hardware_requirements.single_node.full_automation_suite.cpu = 400' ${DEFAULTS_FILE} > "$tmpfile" && \mv "$tmpfile" ${DEFAULTS_FILE}
    jq '.infra.hardware_requirements.single_node.full_automation_suite.memory = 400' ${DEFAULTS_FILE} > "$tmpfile" && \mv "$tmpfile" ${DEFAULTS_FILE}
    jq '.infra.hardware_requirements.multi_node.full_automation_suite.cpu = 400' ${DEFAULTS_FILE} > "$tmpfile" && \mv "$tmpfile" ${DEFAULTS_FILE}
    jq '.infra.hardware_requirements.multi_node.full_automation_suite.memory = 400' ${DEFAULTS_FILE} > "$tmpfile" && \mv "$tmpfile" ${DEFAULTS_FILE}
    
    \cp ${INPUT_FILE} ${INPUT_FILE}.bakup
    jq '.fqdn = "automation-suite.uipath.testing"' ${INPUT_FILE} > "$tmpfile" && \mv "$tmpfile" ${INPUT_FILE}
    jq '.aicenter.enabled = true' ${INPUT_FILE} > "$tmpfile" && \mv "$tmpfile" ${INPUT_FILE}
    jq '.apps.enabled = true' ${INPUT_FILE} > "$tmpfile" && \mv "$tmpfile" ${INPUT_FILE}
    jq '.documentunderstanding.enabled = true' ${INPUT_FILE} > "$tmpfile" && \mv "$tmpfile" ${INPUT_FILE}
    jq '.task_mining.enabled = true' ${INPUT_FILE} > "$tmpfile" && \mv "$tmpfile" ${INPUT_FILE}

    PRE_REQ_OUTPUT=$(${ROOT_DIR}/validateUiPathASReadiness.sh -i ${INPUT_FILE} --install-type online "$@" 2>&1)

    if echo "${PRE_REQ_OUTPUT}" | grep "Machine doesn't have required available CPU to install UiPath Automation Suite. Required CPU should be 400 vCPU";
    then
        echo "Successfully failed with CPU requirements error"
        mark_test_passed 1647364 /tmp/test_results.json
    else
        echo "Could not fail pre-reqs with CPU error"
        mark_test_failed 1647364 /tmp/test_results.json
    fi

    if echo "${PRE_REQ_OUTPUT}" | grep "Machine doesn't have required available RAM to install UiPath Automation Suite. Required RAM should be 360 GiB";
    then
        echo "Successfully failed with Memory requirements error"
        mark_test_passed 1647366 /tmp/test_results.json
    else
        echo "Could not fail pre-reqs with memory error"
        mark_test_failed 1647366 /tmp/test_results.json
    fi

    \cp ${INPUT_FILE}.bakup ${INPUT_FILE}
    \cp ${DEFAULTS_FILE}.bakup ${DEFAULTS_FILE}
}

function validateCorePlatformChecksFailure() {
    tmpfile=$(mktemp)
    \cp ${DEFAULTS_FILE} ${DEFAULTS_FILE}.bakup
    jq '.infra.hardware_requirements.single_node.core_platform.cpu = 400' ${DEFAULTS_FILE} > "$tmpfile" && \mv "$tmpfile" ${DEFAULTS_FILE}
    jq '.infra.hardware_requirements.single_node.core_platform.memory = 400' ${DEFAULTS_FILE} > "$tmpfile" && \mv "$tmpfile" ${DEFAULTS_FILE}
    jq '.infra.hardware_requirements.multi_node.core_platform.cpu = 400' ${DEFAULTS_FILE} > "$tmpfile" && \mv "$tmpfile" ${DEFAULTS_FILE}
    jq '.infra.hardware_requirements.multi_node.core_platform.memory = 400' ${DEFAULTS_FILE} > "$tmpfile" && \mv "$tmpfile" ${DEFAULTS_FILE}
    
    \cp ${INPUT_FILE} ${INPUT_FILE}.bakup
    jq '.fqdn = "automation-suite.uipath.testing"' ${INPUT_FILE} > "$tmpfile" && \mv "$tmpfile" ${INPUT_FILE}
    jq '.aicenter.enabled = false' ${INPUT_FILE} > "$tmpfile" && \mv "$tmpfile" ${INPUT_FILE}
    jq '.apps.enabled = false' ${INPUT_FILE} > "$tmpfile" && \mv "$tmpfile" ${INPUT_FILE}
    jq '.documentunderstanding.enabled = false' ${INPUT_FILE} > "$tmpfile" && \mv "$tmpfile" ${INPUT_FILE}
    jq '.task_mining.enabled = false' ${INPUT_FILE} > "$tmpfile" && \mv "$tmpfile" ${INPUT_FILE}

    systemctl enable firewalld

    PRE_REQ_OUTPUT=$(${ROOT_DIR}/validateUiPathASReadiness.sh -i ${INPUT_FILE} --install-type online "$@" 2>&1)

    if echo "${PRE_REQ_OUTPUT}" | grep "Machine doesn't have required available CPU to install UiPath Automation Suite. Required CPU should be 400 vCPU";
    then
        echo "Successfully failed with CPU requirements error"
        mark_test_passed 1647363 /tmp/test_results.json
    else
        echo "Could not fail pre-reqs with CPU error"
        mark_test_failed 1647363 /tmp/test_results.json
    fi

    if echo "${PRE_REQ_OUTPUT}" | grep "Machine doesn't have required available RAM to install UiPath Automation Suite. Required RAM should be 360 GiB";
    then
        echo "Successfully failed with Memory requirements error"
        mark_test_passed 1647365 /tmp/test_results.json
    else
        echo "Could not fail pre-reqs with memory error"
        mark_test_failed 1647365 /tmp/test_results.json
    fi

    if echo "${PRE_REQ_OUTPUT}" | grep "No IP Address found for automation-suite.uipath.testing";
    then
        echo "Successfully failed with domain resolution error"
        mark_test_passed 1658511 /tmp/test_results.json
    else
        echo "Could not fail with domain reolution error"
        mark_test_passed 1658511 /tmp/test_results.json
    fi

    if echo "${PRE_REQ_OUTPUT}" | grep "Certificate provided is invalid for given FQDN automation-suite.uipath.testing and other sub-domains";
    then
        echo "Successfully failed with certificate SAN error"
    else
        echo "Could not fail with certificate error"
    fi

    if echo "${PRE_REQ_OUTPUT}" | grep "Firewalld is enabled";
    then
        echo "Successfully failed with Firewalld error"
    else
        echo "Could not fail with firewalld error"
    fi

    systemctl disable firewalld
    \cp ${INPUT_FILE}.bakup ${INPUT_FILE}
    \cp ${DEFAULTS_FILE}.bakup ${DEFAULTS_FILE}
}

function main() {
    validateBasicChecksSuccess "$@"
    validateCorePlatformChecksFailure "$@"
    validateFullPlatformChecksFailure "$@"
}

main "$@"